//
//  test23.cpp
//  Lecture3CPP
//
//  Created by presentations on 11/15/17.
//
//


#include <iostream>
#include <iomanip>
using namespace std;

class Point {
    friend ostream &operator << ( ostream &, const Point & );
public:
    Point( int = 0, int = 0 );          // default constructor
    void setPoint( int, int );          // set coordinates
    int getX() const { return x; }     // get x coordinate (inline function)
    int getY() const { return y; }     // get y coordinate (inline function)
protected:                             // accessible by derived classes
    int x, y;                          // x and y coordinates of the Point
}; // end class Point

Point::Point( int a, int b )
{
    setPoint( a, b );
}   // Constructor for class Point

void Point::setPoint( int a, int b )  // Set x and y coordinates of Point
{
    x = a;
    y = b;
} // end function setPoint

// Output Point (with overloaded stream insertion operator)
ostream &operator<<( ostream &output, const Point &p )
{
    output << '[' << p.x << ", " << p.y << ']';
    return output;    // enables cascaded calls
} // end operator<< function


class Circle : public Point {                               // Circle inherits from Point
    friend ostream &operator<<( ostream &, const Circle & );
public:
    Circle( double r = 0.0, int x = 0, int y = 0 );         // default constructor
    void setRadius( double );                                // set radius
    double getRadius() const;                               // return radius
    double area() const;                                        // calculate area
protected:      // accessible by derived classes
    double radius;
}; // end class Circle

// Constructor for Circle calls constructor for Point
// with a member initializer then initializes radius.
Circle::Circle( double r, int a, int b )  :  Point( a, b )       // call base-class constructor
{
    setRadius( r );
}

void Circle::setRadius( double r )
{
    radius = ( r >= 0 ? r : 0 );
} // Set radius of Circle

double Circle::getRadius() const
{
    return radius;
} // Get radius of Circle

double Circle::area() const
{
    return 3.14159 * radius * radius;
}  // Calculate area

// Output a Circle in the form:
// Center = [x, y]; Radius = #.#####
ostream &operator<<( ostream &output, const Circle &c )
{
    output << "Center = " << static_cast< Point >( c ) // '[' << c.x << ", " << c.y << ']'
           << "; Radius = "
           << setiosflags( ios::fixed | ios::showpoint )
           << setprecision( 2 )
           << c.radius;
        
    return output;   // enables cascaded calls
} // end operator<< function
    
    
    
int main()
{
        Point p(2,2);
        Circle c(4, 120, 89);
    
        Point *pointPtr = &p;
        Circle *circlePtr = &c;
    
        cout << "Point p: " << p << "\nCircle c: " << c << endl;
    
        // Treat a Circle as a Point (see only the base class part)
        pointPtr = &c;   // assign address of Circle to pointPtr
        cout << "\nCircle c (via *pointPtr): "  << *pointPtr << endl;
    
        // Treat a Circle as a Circle (with some casting)
        // cast base-class pointer to derived-class pointer
        circlePtr = static_cast< Circle * > ( pointPtr );
        cout << "\nCircle c (via *circlePtr):\n" << *circlePtr << "\nArea of c (via circlePtr): " << circlePtr->area() << endl;
        
        // DANGEROUS: Treat a Point as a Circle
        pointPtr = &p;   // assign address of Point to pointPtr
        
        // cast base-class pointer to derived-class pointer
        circlePtr = static_cast< Circle * > (pointPtr);
        cout << "\nPoint p (via *circlePtr):\n" << *circlePtr << "\nArea of object circlePtr points to: " << circlePtr->area() << endl;
        return 0;
} // end function main
