//
//  test24.cpp
//  Lecture3CPP
//
//  Created by presentations on 11/15/17.
//
//


#include <iostream>
#include <iomanip>
using namespace std;

class Point {
    friend ostream &operator << ( ostream &, const Point & );
public:
    Point( int = 0, int = 0 );          // default constructor
    void setPoint( int, int );          // set coordinates
    int getX() const { return x; }     // get x coordinate (inline function)
    int getY() const { return y; }     // get y coordinate (inline function)
protected:                             // accessible by derived classes
    int x, y;                          // x and y coordinates of the Point
}; // end class Point

Point::Point( int a, int b )
{
    setPoint( a, b );
}   // Constructor for class Point

void Point::setPoint( int a, int b )  // Set x and y coordinates of Point
{
    x = a;
    y = b;
} // end function setPoint

// Output Point (with overloaded stream insertion operator)
ostream &operator<<( ostream &output, const Point &p )
{
    output << '[' << p.x << ", " << p.y << ']';
    return output;    // enables cascaded calls
} // end operator<< function


class Circle : public Point {                               // Circle inherits from Point
    friend ostream &operator<<( ostream &, const Circle & );
public:
    Circle( double r = 0.0, int x = 0, int y = 0 );         // default constructor
    void setRadius( double );                                // set radius
    double getRadius() const;                               // return radius
    double area() const;                                        // calculate area
protected:      // accessible by derived classes
    double radius;
}; // end class Circle

// Constructor for Circle calls constructor for Point
// with a member initializer then initializes radius.
Circle::Circle( double r, int a, int b )  :  Point( a, b )       // call base-class constructor
{
    setRadius( r );
}

void Circle::setRadius( double r )
{
    radius = ( r >= 0 ? r : 0 );
} // Set radius of Circle

double Circle::getRadius() const
{
    return radius;
} // Get radius of Circle

double Circle::area() const
{
    return 3.14159 * radius * radius;
}  // Calculate area

// Output a Circle in the form:
// Center = [x, y]; Radius = #.#####
ostream &operator<<( ostream &output, const Circle &c )
{
    output << "Center = " << static_cast< Point >( c ) // '[' << c.x << ", " << c.y << ']'
    << "; Radius = "
    << setiosflags( ios::fixed | ios::showpoint )
    << setprecision( 2 )
    << c.radius;
    
    return output;   // enables cascaded calls
} // end operator<< function


class Cylinder : public Circle {
    friend ostream &operator<<( ostream &, const Cylinder & );
public:
    // default constructor
    Cylinder( double h = 0.0, double r = 0.0,  int x = 0, int y = 0 );
    
    void setHeight( double );   // set height
    double getHeight() const;  // return height
    double area() const;           // calculate and return area
    double volume() const;      // calculate and return volume
    
protected:
    double height;                    // height of the Cylinder
}; // end class Cylinder


// Cylinder constructor  calls base-class constructor
Cylinder::Cylinder( double h, double r, int x, int y )  : Circle( r, x, y )
{
    setHeight( h );
}

void Cylinder::setHeight( double h ) // Set height of Cylinder
{
    height = ( h >= 0 ? h : 0 );
}

double Cylinder::getHeight() const
{
    return height;
} // Get height of Cylinder

double Cylinder::area() const           // Calculate area of Cylinder (i.e., surface area)
{
    return 2 * Circle::area() + 2 * 3.14159 * radius * height;
} // end function area

double Cylinder::volume() const         // Calculate volume of Cylinder
{
    return Circle::area() * height;
}

ostream &operator<<( ostream &output, const Cylinder &c ) // Output Cylinder dimensions
{
    output << static_cast< Circle >( c )
    << "; Height = " << c.height;
    return output;   // enables cascaded calls
} // end operator<< function
// end class Cylinder (implementation)


int main()
{
    // create Cylinder object
    Cylinder c( 4, 2.5, 2, 23 );
    
    // use get functions to display the Cylinder
    cout << "\nX coordinate is "     << c.getX()
         << "\nY coordinate is "     << c.getY()
         << "\nRadius is "           << c.getRadius()
         << "\nHeight is "           << c.getHeight() << endl;
    
    // use set functions to change the Cylinder's attributes
    c.setHeight( 10 );
    c.setRadius( 4.25 );
    c.setPoint(2, 2);
    cout << "The new location, radius, and height of cylinder are:\n" << c << endl;
    cout << "The area of cylinder is:\n"  << c.area() << endl;
    
    // display the Cylinder as a Point
    Point &pRef = c;   // pRef "thinks" it is a Point
    cout << "\nCylinder printed as a Point is: "  << pRef << "\n\n";
    
    // display the Cylinder as a Circle
    Circle &circleRef = c;  // circleRef thinks it is a Circle
    cout << "Cylinder printed as a Circle is:\n" << circleRef
    << "\nArea: " << circleRef.area() << endl;
    return 0;
} // end function main
