//
//  test28.cpp
//  Lecture3CPP
//
//  Created by presentations on 11/20/17.
//
//

#include <iostream>
#include <iomanip>
using namespace std;

// abstract definition for Shape
class Shape {
public:
    // impure virtual functions
    virtual double area() const { return 0.0; }
    virtual double volume() const { return 0.0; }
    
    // pure virtual functions overridden in each of the derived classes
    virtual void printShapeName() const = 0;
    
    virtual void print() const = 0;
    
    // non–virtual functions/data may be defined, but they are not needed in this class
    
}; // end class Shape


class Point : public Shape {
public:
    Point( int = 0, int = 0 );  // default constructor
    void setPoint( int, int );
    int getX() const { return x; }
    int getY() const { return y; }
    virtual void printShapeName() const { cout << "Point: "; }
    virtual void print() const;
    // area and volume of a point are both 0.0  and they are inherited from Shape
private:
    int x, y;   // x and y coordinates of Point
}; // end class Point


Point::Point( int a, int b ){ setPoint( a, b ); }

void Point::setPoint( int a, int b )
{
    x = a;
    y = b;
} // end function setPoint

void Point::print() const
{ cout << '[' << x << ", " << y << ']'; }


class Circle : public Point {
public:
    // default constructor
    Circle( double r = 0.0, int x = 0, int y = 0 );
    
    void setRadius( double );
    double getRadius() const;
    virtual double area() const;
    virtual void printShapeName() const { cout << "Circle: "; }
    virtual void print() const;
private:
    double radius;   // radius of Circle
}; // end class Circle


Circle::Circle( double r, int a, int b )   : Point( a, b )  // call base-class constructor
{ setRadius( r ); }

void Circle::setRadius( double r )   { radius = r > 0 ? r : 0; }

double Circle::getRadius() const  { return radius; }

double Circle::area() const   { return 3.14159 * radius * radius; }

void Circle::print() const
{
    Point::print();
    cout << "; Radius = " << radius;
} // end function print


class Cylinder : public Circle {
public:
    // default constructor
    Cylinder( double h = 0.0, double r = 0.0, int x = 0, int y = 0 );
    
    void setHeight( double );
    double getHeight();
    virtual double area() const;
    virtual double volume() const;
    virtual void printShapeName() const {cout << "Cylinder: ";}
    virtual void print() const;
private:
    double height;   // height of Cylinder
}; // end class Cylinder


Cylinder::Cylinder( double h, double r, int x, int y ):Circle( r, x, y)//call base-class constructor
{ setHeight( h ); }

double Cylinder::getHeight() { return height; }
void Cylinder::setHeight( double h )   { height = h > 0 ? h : 0; }

double Cylinder::area() const
{
    // surface area of Cylinder
    return 2 * Circle::area() +  2 * 3.14159 * getRadius() * height;
} // end function area

double Cylinder::volume() const     { return Circle::area() * height; }

void Cylinder::print() const
{
    Circle::print();
    cout << "; Height = " << height;
} // end function print


// Make virtual function calls off a base-class pointer using dynamic binding.
void virtualViaPointer( const Shape *baseClassPtr )
{
    baseClassPtr->printShapeName();
    baseClassPtr->print();
    cout<<"\nArea="<<baseClassPtr->area()<<"\nVolume="<< baseClassPtr->volume()<<"\n";
} // end function virtualViaPointer

// Make virtual function calls off a base-class reference using dynamic binding.
void virtualViaReference( const Shape &baseClassRef )
{
    baseClassRef.printShapeName();
    baseClassRef.print();
    cout<<"\nArea="<< baseClassRef.area()<<"\nVolume="<< baseClassRef.volume()<<"\n";
} // end function virtualViaReference


int main()
{
    cout << setiosflags( ios::fixed | ios::showpoint ) << setprecision( 2 );
    
    Point point( 7, 11 );                          // create a Point
    Circle circle( 3.5, 22, 8 );                   // create a Circle
    Cylinder cylinder( 10, 3.3, 10, 10 );          // create a Cylinder
    
    
    point.printShapeName();                        // static binding
    point.print();                                 // static binding
    
    circle.printShapeName();                       // static binding
    circle.print();                                // static binding
    
    cylinder.printShapeName();                     // static binding
    cylinder.print();                              // static binding
    
    Shape *arrayOfShapes[ 3 ];                     // array of base-class pointers
    arrayOfShapes[ 0 ] = &point;                   // aim arrayOfShapes[0] at derived-class Point object
    arrayOfShapes[ 1 ] = &circle;                  // aim arrayOfShapes[1] at derived-class Circle object
    arrayOfShapes[ 2 ] = &cylinder;                // aim arrayOfShapes[2] at derived-class Cylinder object
    
    cout << "\nVirtual function calls made off "  << "base-class pointers\n";
    
    for ( int i = 0; i < 3; i++ )     // Loop through arrayOfShapes and call virtualViaPointer
        virtualViaPointer( arrayOfShapes[ i ] );
    
    cout << "\nVirtual function calls made off " << "base-class references\n";
    
    for ( int j = 0; j < 3; j++ )     // Loop through arrayOfShapes and call virtualViaReference
        virtualViaReference( *arrayOfShapes[ j ] );
    
    return 0;
} // end function main
















